package core

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
)

// CompileIf compiles a branch instruction.
func (f *Function) CompileIf(branch *ast.If) error {
	err := f.Evaluate(branch.Condition)

	if err != nil {
		return err
	}

	endLabel := fmt.Sprintf("%s_end_if_%d", f.Name, f.count.branch)
	f.JumpIfFalse(branch.Condition.Token.Text(), endLabel)
	defer f.assembler.Label(asm.LABEL, endLabel)
	f.count.branch++
	return f.CompileAST(branch.Body)
}

// JumpIfFalse jumps to the label if the previous comparison was false.
func (f *Function) JumpIfFalse(operator string, label string) {
	switch operator {
	case "==":
		f.assembler.Label(asm.JNE, label)
	case "!=":
		f.assembler.Label(asm.JE, label)
	case ">":
		f.assembler.Label(asm.JLE, label)
	case "<":
		f.assembler.Label(asm.JGE, label)
	case ">=":
		f.assembler.Label(asm.JL, label)
	case "<=":
		f.assembler.Label(asm.JG, label)
	}
}

// JumpIfTrue jumps to the label if the previous comparison was true.
func (f *Function) JumpIfTrue(operator string, label string) {
	switch operator {
	case "==":
		f.assembler.Label(asm.JE, label)
	case "!=":
		f.assembler.Label(asm.JNE, label)
	case ">":
		f.assembler.Label(asm.JG, label)
	case "<":
		f.assembler.Label(asm.JL, label)
	case ">=":
		f.assembler.Label(asm.JGE, label)
	case "<=":
		f.assembler.Label(asm.JLE, label)
	}
}
